'ftd2xx.inc for PowerBasic (see www.powerbasic.com)
'FTD2XX.DLL is a native USB device driver for FTDI FT8U232/245 (see www.ftdichip.com)
'
'Include file creator:   Roland Walter,support@rowalt.de
'Last update:            09.02.2003 (DD/MM/YYYY) for FTD2XX.DLL version 2.0.8.1 
'Translation state:      Complete FTD2XX.H translated. Please note that not all functions could be tested.
'
'History (DD.MM.YYY):
'06.02.2003:             Initial release for for FTD2XX.DLL version 1.0.4.1
'09.04.2003:             Fuction FT_GetDeviceInfo added (was mentioned but not described/implemented for earlier FTD2XX.DLL)
'*****************************************************************************************************************************
'Function return codes:
%FT_OK                          = 0
%FT_SUCCESS                     = %FT_OK
%FT_INVALID_HANDLE              = 1
%FT_DEVICE_NOT_FOUND            = 2
%FT_DEVICE_NOT_OPENED           = 3
%FT_IO_ERROR                    = 4
%FT_INSUFFICIENT_RESOURCES      = 5
%FT_INVALID_PARAMETER           = 6
%FT_INVALID_BAUD_RATE           = 7
%FT_DEVICE_NOT_OPENED_FOR_ERASE = 8
%FT_DEVICE_NOT_OPENED_FOR_WRITE = 9
%FT_FAILED_TO_WRITE_DEVICE      = 10
%FT_EEPROM_READ_FAILED          = 11
%FT_EEPROM_WRITE_FAILED         = 12
%FT_EEPROM_ERASE_FAILED         = 13
%FT_EEPROM_NOT_PRESENT          = 14
%FT_EEPROM_NOT_PROGRAMMED       = 15
%FT_INVALID_ARGS                = 16
%FT_OTHER_ERROR                 = 17

'FT_OpenEx flags:
%FT_OPEN_BY_SERIAL_NUMBER  = 1
%FT_OPEN_BY_DESCRIPTION    = 2

'FT_ListDevices flags (used in conjunction with FT_OpenEx flags):
%FT_LIST_NUMBER_ONLY  = &H80000000???
%FT_LIST_BY_INDEX     = &H40000000???
%FT_LIST_ALL          = &H20000000???
%FT_LIST_MASK         = %FT_LIST_NUMBER_ONLY Or %FT_LIST_BY_INDEX Or %FT_LIST_ALL

'FT_GetModemStatus flags (own addition, not declared in FTD2XX.H)
%FT_STATUS_CTS  = &H00000010???   '=&B00010000
%FT_STATUS_DSR  = &H00000020???   '=&B00100000
%FT_STATUS_RING = &H00000040???   '=&B01000000
%FT_STATUS_DCD  = &H00000080???   '=&B10000000

'FT_DEVICE:
%FT_DEVICE_232BM   = 0
%FT_DEVICE_232AM   = 1
%FT_DEVICE_100AX   = 2
%FT_DEVICE_UNKNOWN = 3

'Baud Rates:
%FT_BAUD_300    = 300
%FT_BAUD_600    = 600
%FT_BAUD_1200   = 1200
%FT_BAUD_2400   = 2400
%FT_BAUD_4800   = 4800
%FT_BAUD_9600   = 9600
%FT_BAUD_14400  = 14400
%FT_BAUD_19200  = 19200
%FT_BAUD_38400  = 38400
%FT_BAUD_57600  = 57600
%FT_BAUD_115200 = 115200
%FT_BAUD_230400 = 230400
%FT_BAUD_460800 = 460800
%FT_BAUD_921600 = 921600

'Word Lengths:
%FT_BITS_8 = 8
%FT_BITS_7 = 7

'Stop Bits:
%FT_STOP_BITS_1   = 0
%FT_STOP_BITS_1_5 = 1
%FT_STOP_BITS_2   = 2

'Parity:
%FT_PARITY_NONE  = 0
%FT_PARITY_ODD   = 1
%FT_PARITY_EVEN  = 2
%FT_PARITY_MARK  = 3
%FT_PARITY_SPACE = 4

'Flow Control:
%FT_FLOW_NONE     = &H000
%FT_FLOW_RTS_CTS  = &H100
%FT_FLOW_DTR_DSR  = &H200
%FT_FLOW_XON_XOFF = &H400

'Purge rx and tx buffers:
%FT_PURGE_RX = 1
%FT_PURGE_TX = 2

'Events:
%FT_EVENT_RXCHAR        = 1
%FT_EVENT_MODEM_STATUS  = 2

'Timeouts:
%FT_DEFAULT_RX_TIMEOUT  = 300
%FT_DEFAULT_TX_TIMEOUT  = 300

'Structure to hold program data for FT_Program function:
Type FT_PROGRAM_DATA
  VendorId As Word                 'Default: &H0403 (if other value: change INF file and ask FTDI for further advices)
  ProductId As Word                'Default: &H6001 (if other value: change INF file and ask FTDI for further advices)
  Manufacturer As Asciiz Ptr       'max 32 bytes; default: "FTDI"
  ManufacturerId As Asciiz Ptr     'max 16 bytes; default: "FT"
  Description As Asciiz Ptr        'max 64 bytes; default: "USB HS Serial Converter"
  SerialNumber As Asciiz Ptr       'max 16 bytes; default: "" (NULL); set to "FT000001" for example for fixed serial number
  MaxPower As Word                 '0 < MaxPower <= 500; default: 44
  PnP As Word                      '0 = disabled, 1 = enabled; default: 1
  SelfPowered As Word              '0 = bus powered, 1 = self powered; default: 0
  RemoteWakeup As Word             '0 = not capable, 1 = capable; default: 1
  'Rev4 extensions:
  Rev4 As Byte                     'TRUE if Rev4 chip, false otherwise; default: FALSE
  IsoIn As Byte                    'TRUE if in endpoint is isochronous; default: FALSE
  IsoOut As Byte                   'TRUE if out endpoint is isochronous; default: FALSE
  PullDownEnable As Byte           'TRUE if pull down enabled; default: FALSE
  SerNumEnable As Byte             'TRUE if serial number to be used; default: FALSE
  USBVersionEnable As Byte         'TRUE if chip uses USBVersion; default: FALSE
  USBVersion As Word               'BCD (&H0200 => USB2); default: 0
End Type

'Structures for the Win32 COMM API type functions:

Type FTCOMSTAT         'Same as Win32 API COMSTAT
  fbits As Dword       'Bit 0: fCtsHold
                       'Bit 1: fDsrHold
                       'Bit 2: fRlsdHold
                       'Bit 3: fXoffHold
                       'Bit 4: fXoffSent
                       'Bit 5: fEof
                       'Bit 6: fTxim
  cbInQue As Dword
  cbOutQue As Dword
End Type

Type FTDCB             'Same as Win32 API DCB
  DCBlength As Dword   'SizeOf(FTDCB)
  BaudRate As Dword    'Baudrate at which running
  fBits As Dword       'Bit  0: fBinary
                       'Bit  1: fParity
                       'Bit  2: fOutxDtsFlow
                       'Bit  3: fOutxDsrFlow
                       'Bit  4: fDtrControl
                       'Bit  5: fDsrSensitivity
                       'Bit  6: fTXContinueOnXoff
                       'Bit  7: fOutX
                       'Bit  8: fInX
                       'Bit  9: fErrorChar
                       'Bit 10: fNull
                       'Bit 11: fRtsControl
                       'Bit 12: fAbortOnError
                       'Bit 13: fDummy2
  wReserved As Word    'Not currently used
  XonLim As Word       'Transmit X-ON threshold
  XoffLim As Word      'Transmit X-OFF threshold
  ByteSize As Byte     'Number of bits/byte, 4-8
  Parity As Byte       '0-4=None,Odd,Even,Mark,Space
  StopBits As Byte     '0,1,2 = 1, 1.5, 2
  XonChar As Byte      'Tx and Rx X-ON character
  XoffChar As Byte     'Tx and Rx X-OFF character
  ErrorChar As Byte    'Error replacement char
  EofChar As Byte      'End of Input character
  EvtChar As Byte      'Received Event character
  wReserved2 As Word   'Fill for now
End Type

Type FTTIMEOUTS                         'Same as Win32 API COMMTIMEOUTS
  ReadIntervalTimeout As Dword          'Maximum time between read chars
  ReadTotalTimeoutMultiplier As Dword   'Multiplier of characters
  ReadTotalTimeoutConstant As Dword     'Constant in milliseconds
  WriteTotalTimeoutMultiplier As Dword  'Multiplier of characters
  WriteTotalTimeoutConstant As Dword    'Constant in milliseconds
End Type

'Function exports:
Declare Function FT_Open Lib "FTD2XX.DLL" Alias "FT_Open" (ByVal iDeviceNumber As Integer,hDevice As Dword) As Long
Declare Function FT_Close Lib "FTD2XX.DLL" Alias "FT_Close" (ByVal hDevice As Dword) As Long
Declare Function FT_Read Lib "FTD2XX.DLL" Alias "FT_Read" (ByVal hDevice As Dword,lpBuffer As Any,ByVal dwBytesToRead As Dword,pdwBytesReturned As Dword) As Long
Declare Function FT_Write Lib "FTD2XX.DLL" Alias "FT_Write" (ByVal hDevice As Dword,lpBuffer As Any,ByVal dwBytesToWrite As Long,lpdwBytesWritten As Dword) As Long
Declare Function FT_IoCtl Lib "FTD2XX.DLL" Alias "FT_IoCtl" (ByVal hDevice As Dword,ByVal dwIoControlCode As Dword,lpInBuf As Any,ByVal nInBufSize As Dword,lpOutBuf As Any,nOutBufSize As Dword,lpBytesReturned As Dword,lpOverlapped As OVERLAPPED) As Long
Declare Function FT_ResetDevice Lib "FTD2XX.DLL" Alias "FT_ResetDevice" (ByVal hDevice As Dword) As Long
Declare Function FT_SetBaudRate Lib "FTD2XX.DLL" Alias "FT_SetBaudRate" (ByVal hDevice As Dword,ByVal dwBaudRate As Dword) As Long
Declare Function FT_SetDataCharacteristics Lib "FTD2XX.DLL" Alias "FT_SetDataCharacteristics" (ByVal hDevice As Dword,ByVal byWordLength As Byte,ByVal byStopBits As Byte,ByVal byParity As Byte) As Long
Declare Function FT_SetFlowControl Lib "FTD2XX.DLL" Alias "FT_SetFlowControl" (ByVal hDevice As Dword,ByVal usFlowControl As Word,ByVal byXonChar As Byte,ByVal byXoffChar As Byte) As Long
Declare Function FT_SetDtr Lib "FTD2XX.DLL" Alias "FT_SetDtr" (ByVal hDevice As Dword) As Long
Declare Function FT_ClrDtr Lib "FTD2XX.DLL" Alias "FT_ClrDtr" (ByVal hDevice As Dword) As Long
Declare Function FT_SetRts Lib "FTD2XX.DLL" Alias "FT_SetRts" (ByVal hDevice As Dword) As Long
Declare Function FT_ClrRts Lib "FTD2XX.DLL" Alias "FT_ClrRts" (ByVal hDevice As Dword) As Long
Declare Function FT_GetModemStatus Lib "FTD2XX.DLL" Alias "FT_GetModemStatus" (ByVal hDevice As Dword,dwModemStatus As Dword) As Long
Declare Function FT_SetChars Lib "FTD2XX.DLL" Alias "FT_SetChars" (ByVal hDevice As Dword,ByVal byEventChar As Byte,ByVal byEventCharEnabled As Byte,ByVal byErrorChar As Byte,ByVal byErrorCharEnabled As Byte) As Long
Declare Function FT_Purge Lib "FTD2XX.DLL" Alias "FT_Purge" (ByVal hDevice As Dword,ByVal dwMask As Dword) As Long
Declare Function FT_SetTimeouts Lib "FTD2XX.DLL" Alias "FT_SetTimeouts" (ByVal hDevice As Dword,ByVal dwReadTimeout As Dword,ByVal dwWriteTimeout As Dword) As Long
Declare Function FT_GetQueueStatus Lib "FTD2XX.DLL" Alias "FT_GetQueueStatus" (ByVal hDevice As Dword,pdwAmountInRxQueue As Dword) As Long
Declare Function FT_SetEventNotification Lib "FTD2XX.DLL" Alias "FT_SetEventNotification" (ByVal hDevice As Dword,ByVal dwEventMask As Dword,pvArg As Long) As Long
Declare Function FT_GetEventStatus Lib "FTD2XX.DLL" Alias "FT_GetEventStatus" (ByVal hDevice As Dword,dwEventsDword As Dword) As Long
Declare Function FT_GetStatus Lib "FTD2XX.DLL" Alias "FT_GetStatus" (ByVal hDevice As Dword,lpdwAmountInRxQueue As Dword,lpdwAmountInTxQueue As Dword,lpdwEventStatus As Dword) As Long
Declare Function FT_SetBreakOn Lib "FTD2XX.DLL" Alias "FT_SetBreakOn" (ByVal hDevice As Dword) As Long
Declare Function FT_SetBreakOff Lib "FTD2XX.DLL" Alias "FT_SetBreakOff" (ByVal hDevice As Dword) As Long
Declare Function FT_SetWaitMask Lib "FTD2XX.DLL" Alias "FT_SetWaitMask" (ByVal hDevice As Dword,ByVal dwMask As Dword) As Long
Declare Function FT_WaitOnMask Lib "FTD2XX.DLL" Alias "FT_WaitOnMask" (ByVal hDevice As Dword,dwMask As Dword) As Long
Declare Function FT_SetDivisor Lib "FTD2XX.DLL" Alias "FT_SetDivisor" (ByVal hDevice As Dword,ByVal wDivisor As Word) As Long
Declare Function FT_OpenEx Lib "FTD2XX.DLL" Alias "FT_OpenEx" (pvArg1 As Any,ByVal dwFlags As Dword,lpdwHandle As Dword) As Long
Declare Function FT_ListDevices Lib "FTD2XX.DLL" Alias "FT_ListDevices" (pvArg1 As Any,pvArg2 As Any,ByVal dwFlags As Dword) As Long
Declare Function FT_SetLatencyTimer Lib "FTD2XX.DLL" Alias "FT_SetLatencyTimer" (ByVal hDevice As Dword,ByVal ucTimer As Byte) As Long
Declare Function FT_GetLatencyTimer Lib "FTD2XX.DLL" Alias "FT_GetLatencyTimer" (ByVal hDevice As Dword,ucTimer As Byte) As Long
Declare Function FT_SetBitMode Lib "FTD2XX.DLL" Alias "FT_SetBitMode" (ByVal hDevice As Dword,ByVal ucMask As Byte,ByVal ucEnable As Byte) As Long
Declare Function FT_GetBitMode Lib "FTD2XX.DLL" Alias "FT_GetBitMode" (ByVal hDevice As Dword,pucMode As Byte) As Long
Declare Function FT_SetUSBParameters Lib "FTD2XX.DLL" Alias "FT_SetUSBParameters" (ByVal hDevice As Dword,ByVal dwInTransferSize As Dword,ByVal dwOutTransferSize As Dword) As Long
Declare Function FT_GetDeviceInfo Lib "FTD2XX.DLL" Alias "FT_GetDeviceInfo" (ByVal hDevice As Dword,pftType As Dword,lpdwID As Dword,pcSerialNumber As Dword,pcDescription As Dword,pvDummy As Dword) As Long
Declare Function FT_EraseEE Lib "FTD2XX.DLL" Alias "FT_EraseEE" (ByVal hDevice As Dword) As Long
Declare Function FT_ReadEE Lib "FTD2XX.DLL" Alias "FT_ReadEE" (ByVal hDevice As Dword,ByVal dwWordOffset As Dword,lpwValue As Word) As Long
Declare Function FT_WriteEE Lib "FTD2XX.DLL" Alias "FT_WriteEE" (ByVal hDevice As Dword,ByVal dwWordOffsetDword As Dword,ByVal wValue As Word) As Long
Declare Function FT_EE_Program Lib "FTD2XX.DLL" Alias "FT_EE_Program" (ByVal hDevice As Dword, lpData As FT_PROGRAM_DATA) As Long
Declare Function FT_EE_Read Lib "FTD2XX.DLL" Alias "FT_EE_Read" (ByVal hDevice As Dword, lpData As FT_PROGRAM_DATA) As Long
Declare Function FT_EE_UARead Lib "FTD2XX.DLL" Alias "FT_EE_UARead" (ByVal hDevice As Dword, pucData As Any, ByVal dwDataLen As Long, lpdwBytesRead As Long) As Long
Declare Function FT_EE_UASize Lib "FTD2XX.DLL" Alias "FT_EE_UASize" (ByVal hDevice As Dword, lpdwSize As Long) As Long
Declare Function FT_EE_UAWrite Lib "FTD2XX.DLL" Alias "FT_EE_UAWrite" (ByVal hDevice As Dword, pucData As Any, ByVal dwDataLen As Long) As Long
Declare Function FT_W32_CreateFile Lib "FTD2XX.DLL" Alias "FT_W32_CreateFile" (lpFileName As Asciiz,ByVal dwDesiredAccess As Dword,_
                ByVal dwShareMode As Dword,lpSecurityAttributes As SECURITY_ATTRIBUTES,ByVal dwCreationDisposition As Dword,ByVal dwFlagsAndAttributes As Dword,ByVal hTemplateFile As Dword) As Long
Declare Function FT_W32_CloseHandle Lib "FTD2XX.DLL" Alias "FT_W32_CloseHandle" (ByVal hDevice As Dword) As Long
Declare Function FT_W32_ReadFile Lib "FTD2XX.DLL" Alias "FT_W32_ReadFile" (ByVal hDevice As Dword,lpBuffer As Any,ByVal dwBytesToRead As Dword,lpdwBytesReturned As Dword,lpOverlapped As OVERLAPPED) As Long
Declare Function FT_W32_WriteFile Lib "FTD2XX.DLL" Alias "FT_W32_WriteFile" (ByVal hDevice As Dword,lpBuffer As Any,ByVal dwBytesToWrite  As Dword,lpdwBytesWritten As Dword,lpOverlapped As OVERLAPPED) As Long
Declare Function FT_W32_GetOverlappedResult Lib "FTD2XX.DLL" Alias "FT_W32_GetOverlappedResult" (ByVal hDevice As Dword,lpOverlapped As OVERLAPPED,pdwBytesTransferred As Dword,ByVal bWait As Long) As Long
Declare Function FT_W32_ClearCommBreak Lib "FTD2XX.DLL" Alias "FT_W32_ClearCommBreak" (ByVal hDevice As Dword) As Long
Declare Function FT_W32_ClearCommError Lib "FTD2XX.DLL" Alias "FT_W32_ClearCommError" (ByVal hDevice As Dword,lpdwErrors As Dword,lpftComstat As COMSTAT) As Long
Declare Function FT_W32_EscapeCommFunction Lib "FTD2XX.DLL" Alias "FT_W32_EscapeCommFunction" (ByVal hDevice As Dword,ByVal dwFunc As Dword) As Long
Declare Function FT_W32_GetCommModemStatus Lib "FTD2XX.DLL" Alias "FT_W32_GetCommModemStatus" (ByVal hDevice As Dword,lpdwStat As Dword) As Long
Declare Function FT_W32_GetCommState Lib "FTD2XX.DLL" Alias "FT_W32_GetCommState" (ByVal hDevice As Dword,lpDCB As FTDCB) As Long
Declare Function FT_W32_GetCommTimeouts Lib "FTD2XX.DLL" Alias "FT_W32_GetCommTimeouts" (ByVal hDevice As Dword,lpCommTimeouts As FTTIMEOUTS) As Long
Declare Function FT_W32_GetLastError Lib "FTD2XX.DLL" Alias "FT_W32_GetLastError" (ByVal hDevice As Dword) As Long
Declare Function FT_W32_PurgeComm Lib "FTD2XX.DLL" Alias "FT_W32_PurgeComm" (ByVal hDevice As Dword,ByVal dwFlags As Dword) As Long
Declare Function FT_W32_SetCommBreak Lib "FTD2XX.DLL" Alias "FT_W32_SetCommBreak" (ByVal hDevice As Dword) As Long
Declare Function FT_W32_SetCommMask Lib "FTD2XX.DLL" Alias "FT_W32_SetCommMask" (ByVal hDevice As Dword,ByVal dwMask As Dword) As Long
Declare Function FT_W32_SetCommState Lib "FTD2XX.DLL" Alias "FT_W32_SetCommState" (ByVal hDevice As Dword,lpDCB As FTDCB) As Long
Declare Function FT_W32_SetCommTimeouts Lib "FTD2XX.DLL" Alias "FT_W32_SetCommTimeouts" (ByVal hDevice As Dword,lpCommTimeouts As FTTIMEOUTS) As Long
Declare Function FT_W32_SetupComm Lib "FTD2XX.DLL" Alias "FT_W32_SetupComm" (ByVal hDevice As Dword, ByVal dwReadBufferSize As Dword, ByVal dwWriteBufferSize As Dword) As Long
Declare Function FT_W32_WaitCommEvent Lib "FTD2XX.DLL" Alias "FT_W32_WaitCommEvent" (ByVal hDevice As Dword,  lpdwEvent As Dword, lpOverlapped As OVERLAPPED) As Long
